package com.huiju.estateagents.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huiju.estateagents.base.ResponseBean;
import com.huiju.estateagents.center.taUser.entity.TaUser;
import com.huiju.estateagents.center.taUser.service.ITaUserService;
import com.huiju.estateagents.common.CommConstant;
import com.huiju.estateagents.common.DateUtils;
import com.huiju.estateagents.common.StringUtils;
import com.huiju.estateagents.entity.*;
import com.huiju.estateagents.excel.AgentsRecommendCustomer;
import com.huiju.estateagents.excel.ExcelRecommendCustomer;
import com.huiju.estateagents.excel.ReporRecommendCustomer;
import com.huiju.estateagents.mapper.*;
import com.huiju.estateagents.service.IMiniAppService;
import com.huiju.estateagents.service.ITaRecommendCustomerService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huiju.estateagents.statistic.entity.TaCustomerFollowUpRecord;
import com.huiju.estateagents.statistic.mapper.TaCustomerFollowUpRecordMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 推荐客户表 服务实现类
 * </p>
 *
 * @author jobob
 * @since 2019-05-10
 */
@Service
public class TaRecommendCustomerServiceImpl extends ServiceImpl<TaRecommendCustomerMapper, TaRecommendCustomer> implements ITaRecommendCustomerService {

    @Autowired
    TaPersonMapper taPersonMapper;

    @Autowired
    TaRecommendCustomerMapper taRecommendCustomerMapper;

    @Autowired
    IMiniAppService iMiniAppService;

    @Autowired
    TaPersonVisitRecordMapper taPersonVisitRecordMapper;

    @Autowired
    ITaUserService userService;

    @Autowired
    private TaBuildingMapper taBuildingMapper;

    @Autowired
    private TdCityMapper cityMapper;
    
    @Autowired
    private TaCustomerFollowUpRecordMapper customerFollowUpRecordMapper;

    @Override
    public ResponseBean getMyCustList(String openid, String keywords, int pageNumber, int pageSize) {
        List<TaPerson> taPersons = getPersonsByOpenId(openid);
        if (null == taPersons || taPersons.size() != 1) {
            return ResponseBean.error("验证当前人员信息失败", ResponseBean.ERROR_UNAVAILABLE);
        }

        TaPerson taPerson = taPersons.get(0);

        IPage<TaRecommendCustomer> page = new Page<>();
        page.setCurrent(pageNumber);
        page.setSize(pageSize);

        QueryWrapper<TaRecommendCustomer> query = new QueryWrapper<>();
        query.eq("recommend_person", taPerson.getPersonId());
        query.like(null != keywords && "".equals(keywords.trim()), "name", keywords);
        query.orderByDesc("report_date");

        IPage<TaRecommendCustomer> result = taRecommendCustomerMapper.selectPage(page, query);
        return ResponseBean.success(result);
    }

    @Override
    public ResponseBean getCustDetail(String id) {
        TaRecommendCustomer taRecommendCustomer = taRecommendCustomerMapper.selectById(id);

        if (!StringUtils.isEmpty(taRecommendCustomer.getRealtyConsultant())) {
            TaPerson taPerson = taPersonMapper.selectById(taRecommendCustomer.getRealtyConsultant());
            if (null != taPerson) {
                TaUser user = userService.getById(taPerson.getUserId());
                taRecommendCustomer.setConsultants(new HashMap<String, Object>() {{
                    put("name", user.getUserName());
                    put("phone", user.getPhone());
                    put("photo", user.getPhoto());
                }});
            }
        }

        return ResponseBean.success(taRecommendCustomer);
    }

    @Override
    public ResponseBean updateCustByJSON(String paramStr) {
        JSONObject params = JSONObject.parseObject(paramStr);
        if (null == params) {
            return ResponseBean.error("解析参数失败", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        String checkResult = checkCustomerParams(params);
        if (null != checkResult) {
            return ResponseBean.error(checkResult, ResponseBean.ERROR_MISSING_PARAMS);
        }


        UpdateWrapper<TaRecommendCustomer> wrapper = new UpdateWrapper<>();
        wrapper.eq("customer_id", params.get("id"));
        wrapper.set("name", params.get("name"));
        wrapper.set("picture", params.get("picture"));
        wrapper.set("phone", params.get("phone"));
        wrapper.set("sex", params.get("sex"));
        wrapper.set("describe", params.get("describe"));

        String at = params.getString("appointmentTime");
        if (!StringUtils.isEmpty(at)) {
            wrapper.set("appointmentTime", DateUtils.day2LocalDateime(at));
        }
        wrapper.set("vistors", params.get("vistors"));
        wrapper.set("explain", params.get("explain"));
        wrapper.set("intention", params.get("intention"));
        wrapper.set("realtyManageType", params.get("realtyManageType"));
        wrapper.set("demandType", params.get("demandType"));
        wrapper.set("priceRange", params.get("priceRange"));
        wrapper.set("realtyConsultant", params.get("realtyConsultant"));

        // todo 需要约定时间格式
//        wrapper.set("reportDate", params.get("reportDate"));

        String status = params.getString("status");
        if (null != status) {
            wrapper.set("status", params.get("status"));
        }

        int row = taRecommendCustomerMapper.update(new TaRecommendCustomer(), wrapper);
        if (row <= 0) {
            return ResponseBean.error("更新客户信息失败", ResponseBean.ERROR_UNAVAILABLE);
        }

        return ResponseBean.success("更新客户信息成功");
    }

    @Override
    public ResponseBean recommendCust(String paramStr, String openid,Integer orgId) {
        JSONObject params = JSONObject.parseObject(paramStr);
        if (null == params) {
            return ResponseBean.error("解析参数失败", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }
        List<TaPerson> taPersons = getPersonsByOpenId(openid);
        if (null == taPersons || taPersons.size() != 1) {
            return ResponseBean.error("验证当前人员信息失败", ResponseBean.ERROR_UNAVAILABLE);
        }
        TaPerson taPerson = taPersons.get(0);

        String checkResult = checkCustomerParams(params);
        if (null != checkResult) {
            return ResponseBean.error(checkResult, ResponseBean.ERROR_MISSING_PARAMS);
        }

        TaRecommendCustomer taRecommendCustomer = new TaRecommendCustomer();

        taRecommendCustomer.setName(params.getString("name"));
        taRecommendCustomer.setPicture(params.getString("picture"));
        taRecommendCustomer.setPhone(params.getString("phone"));
        taRecommendCustomer.setSex(params.getInteger("sex"));
        taRecommendCustomer.setDescribe(params.getString("describe"));
        taRecommendCustomer.setRecommendPerson(params.getString("recommendPerson"));
        taRecommendCustomer.setBuildingId(params.getString("intention"));
        taRecommendCustomer.setRealtyConsultant(params.getString("realtyConsultant"));
        taRecommendCustomer.setReportRecommendStatus(CommConstant.RECOMMENDED);
        taRecommendCustomer.setVerifyStatus(CommConstant.VERIFY_READY);
        taRecommendCustomer.setCreateDate(LocalDateTime.now());
        taRecommendCustomer.setReportDate(LocalDateTime.now());
        taRecommendCustomer.setOrgId(orgId);
       QueryWrapper<TaPerson> personQueryWrapper = new QueryWrapper<>();
       personQueryWrapper.eq("org_id",orgId);
       personQueryWrapper.eq("phone",params.getString("phone"));
        TaPerson currentPerson = taPersonMapper.selectOne(personQueryWrapper);
        if(null != currentPerson){
            taRecommendCustomer.setPersonId(currentPerson.getPersonId());
        }

        // 时间格式 yyyy-MM-dd
        String at = params.getString("appointmentTime");
        if (!StringUtils.isEmpty(at)) {
            taRecommendCustomer.setAppointmentTime(DateUtils.dayToLocalDateime(at));
        }

//        taRecommendCustomer.setAppointmentTime((String) params.get("appointmentTime"));
        taRecommendCustomer.setVisiteNum(params.getInteger("visiteNum"));

        TaBuilding building = taBuildingMapper.selectById(params.getString("intention"));
        taRecommendCustomer.setIntention(building.getBuildingName());

        taRecommendCustomer.setRealtyManageType(params.getString("realtyManageType"));
        taRecommendCustomer.setDemandType(params.getString("demandType"));
        taRecommendCustomer.setPriceRange(params.getString("priceRange"));
        taRecommendCustomer.setEntryType(CommConstant.ENTRY_INPUT);
        // taRecommendCustomer.setVerifyStatus(CommConstant.VERIFY_READY);


        String status = params.getString("status");
        int statusInt = 0;
        if (StringUtils.isEmpty(status)){
            taRecommendCustomer.setStatus(statusInt);
        }else{
            taRecommendCustomer.setStatus(Integer.parseInt(status));
        }

        taRecommendCustomer.setRecommendPerson(taPerson.getPersonId());

        // 报备有 10 天有效期
        QueryWrapper<TaRecommendCustomer> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("phone", taRecommendCustomer.getPhone());
        queryWrapper.eq("building_id",taRecommendCustomer.getBuildingId());
        queryWrapper.in("verify_status",CommConstant.VERIFY_AGREE,CommConstant.VERIFY_READY);
        List<TaRecommendCustomer> result = taRecommendCustomerMapper.selectList(queryWrapper);
        if (null != result && result.size() > 0) {
            return ResponseBean.error("当前客户已被推荐, 不能再次推荐", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        int row = taRecommendCustomerMapper.insert(taRecommendCustomer);
        if (row < 1) {
            return ResponseBean.error("推荐客户失败", ResponseBean.ERROR_UNAVAILABLE);
        }

//        // 找到销售经理，并推送消息
//        TaPerson saleMan = taPersonMapper.getSalesExecutive();
//        if (null != saleMan && !StringUtils.isEmpty(saleMan.getMiniOpenid())) {
//            // 发送消息
//            iMiniAppService.sendNewCustomerMessage(saleMan.getMiniOpenid(), CommConstant.MINIAPP_INDEX, taRecommendCustomer.getName(), taRecommendCustomer.getPhone(), "", taPerson.getName());
//        }

        return ResponseBean.success(taRecommendCustomer);
    }



    @Override
    public ResponseBean reportCust(String paramStr, String openid,Integer orgId) {
        JSONObject params = JSONObject.parseObject(paramStr);
        if (null == params) {
            return ResponseBean.error("解析参数失败", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }
        List<TaPerson> taPersons = getPersonsByOpenId(openid);
        if (null == taPersons || taPersons.size() != 1) {
            return ResponseBean.error("验证当前人员信息失败", ResponseBean.ERROR_UNAVAILABLE);
        }
        TaPerson taPerson = taPersons.get(0);

//        String checkResult = checkCustomerParams(params);
//        if (null != checkResult) {
//            return ResponseBean.error(checkResult, ResponseBean.ERROR_MISSING_PARAMS);
//        }

        TaRecommendCustomer taRecommendCustomer = new TaRecommendCustomer();
        taRecommendCustomer.setOrgId(orgId);
        taRecommendCustomer.setName(taPerson.getNickname());
        taRecommendCustomer.setPicture(taPerson.getAvatarurl());
        taRecommendCustomer.setPhone(params.getString("phone"));
        taRecommendCustomer.setSex(null == taPerson.getGender() ? null : ("1".equals(taPerson.getGender()) ? 1 : 2));
//        taRecommendCustomer.setDescribe(params.getString("describe"));
        taRecommendCustomer.setRealtyConsultant(params.getString("realtyConsultant"));
        taRecommendCustomer.setBuildingId(params.getString("buildingId"));
        TaBuilding building = taBuildingMapper.selectById(params.getString("buildingId"));
        taRecommendCustomer.setIntention(building.getBuildingName());
        taRecommendCustomer.setReportRecommendStatus(CommConstant.VERIFY_AGREE);
        taRecommendCustomer.setCreateDate(LocalDateTime.now());
        taRecommendCustomer.setReportDate(LocalDateTime.now());
        taRecommendCustomer.setPersonId(taPerson.getPersonId());

        // 时间格式 yyyy-MM-dd
//        String at = params.getString("appointmentTime");
//        if (!StringUtils.isEmpty(at)) {
//            taRecommendCustomer.setAppointmentTime(DateUtils.day2LocalDateime(at));
//        }ni

//        taRecommendCustomer.setAppointmentTime((String) params.get("appointmentTime"));
//        taRecommendCustomer.setVisiteNum(params.getInteger("visiteNum"));
//        taRecommendCustomer.setIntention(params.getString("intention"));
//        taRecommendCustomer.setRealtyManageType(params.getString("realtyManageType"));
//        taRecommendCustomer.setDemandType(params.getString("demandType"));
//        taRecommendCustomer.setPriceRange(params.getString("priceRange"));
        taRecommendCustomer.setEntryType(CommConstant.ENTRY_VERIFY);
        taRecommendCustomer.setVerifyStatus(CommConstant.VERIFY_AGREE);
        taRecommendCustomer.setStatus(CommConstant.CUSTOMER_REPORT);
        QueryWrapper<TaPerson> personQueryWrapper = new QueryWrapper<>();
        personQueryWrapper.eq("org_id",orgId);
        personQueryWrapper.eq("phone",params.getString("phone"));
        TaPerson currentPerson = taPersonMapper.selectOne(personQueryWrapper);
        if(null != currentPerson){
            taRecommendCustomer.setPersonId(currentPerson.getPersonId());
        }


//        String status = params.getString("status");
//        if (null == status) status = "1";
//        taRecommendCustomer.setStatus(status);
//        taRecommendCustomer.setRecommendPerson(taPerson.getPersonId());
        taRecommendCustomer.setRecommendPerson(params.getString("realtyConsultant"));

        // 报备有 10 天有效期
        QueryWrapper<TaRecommendCustomer> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("name", taRecommendCustomer.getName());
        queryWrapper.eq("phone", taRecommendCustomer.getPhone());
        queryWrapper.eq("building_id",taRecommendCustomer.getBuildingId());
        queryWrapper.in("verify_status",CommConstant.VERIFY_AGREE,CommConstant.VERIFY_READY);
        List<TaRecommendCustomer> result = taRecommendCustomerMapper.selectList(queryWrapper);
        if (null != result && result.size() > 0) {
            return ResponseBean.error("当前客户已被报备, 不能再次报备", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        int row = taRecommendCustomerMapper.insert(taRecommendCustomer);
        if (row < 1) {
            return ResponseBean.error("报备客户失败", ResponseBean.ERROR_UNAVAILABLE);
        }

        // 找到销售经理，并推送消息
        TaPerson saleMan = taPersonMapper.getSalesExecutive();
        if (null != saleMan && !StringUtils.isEmpty(saleMan.getMiniOpenid())) {
            // 发送消息
            iMiniAppService.sendNewCustomerMessage(saleMan, CommConstant.MINIAPP_INDEX, taRecommendCustomer.getName(), taRecommendCustomer.getPhone(), "", taPerson.getName());
        }

        return ResponseBean.success(taRecommendCustomer);
    }

    String checkCustomerParams(JSONObject params) {
        if (null == params.getString("name")) {
            return "请设置客户姓名";
        }
        if (null == params.getString("phone")) {
            return "请设置客户电话";
        }
        if (null == params.getInteger("sex")) {
            return "请设置客户性别";
        }
        if (null == params.getString("intention")) {
            return "请设置意向楼盘";
        }

        return null;
    }




    List<TaPerson> getPersonsByOpenId(String openid) {
        return taPersonMapper.getPersonByOpenId(openid);
    }

    @Override
    public IPage<TaRecommendCustomer> getUnSignedCustomerList(int pageNumber, int pageSize,String building,String personId){
        QueryWrapper<TaRecommendCustomer>queryWrapper = new QueryWrapper<>();
//        queryWrapper.eq("building_id",building);
        queryWrapper.eq("realty_consultant",personId);
        queryWrapper.gt("status", CommConstant.MESSAGE_UNREAD);
        IPage<TaRecommendCustomer>page = new Page<>(pageNumber,pageSize);
        return taRecommendCustomerMapper.selectPage(page,queryWrapper);
    }

    @Override
    public IPage getCustomerList(Integer status,int pageNumber, int pageSize,String building,String name,String tel,String consultName,String consultTel,String entryType,String verifyStatus,Integer sex, String customerType, Integer orgId){

        if (CommConstant.CUTOMER_TYPE_PUBLIC.equals(customerType)) {

            /**
             * <if test="buildingId != null">
             *             ,(select tpp.name from ta_share_person_from tspf left join ta_person tpp on  tspf.share_person = tpp.person_id where tspf.building_id = #{buildingId} and tspf.is_first_time = 1) as sharePersonName
             *         </if>
             */

            /**
             * SELECT
             *         b.*
             *         FROM
             *         ta_person b
             *         <where>
             *             b.status > 0
             *             AND IFNULL(b.person_type, '') in ('estate agent', 'customer')
             *             <if test="name != null and name !=''">
             *                 and b.nickname like CONCAT('%',#{name}, '%')
             *             </if>
             *             <if test="tel != null and tel!=''">
             *                 and b.phone like CONCAT('%',#{tel}, '%')
             *             </if>
             *             <if test="entryType != null and entryType !=''">
             *                 and b.entry_type = #{entryType}
             *             </if>
             *             <if test="sex != null and sex !=''">
             *                 and b.sex = #{sex}
             *             </if>
             *             <if test="orgId != null">
             *                 and b.org_id = #{orgId}
             *             </if>
             *             <if test="consultTel != null and consultTel !=''">
             *                 and b.tel like CONCAT('%',#{consultTel}, '%')
             *             </if>
             *             and NOT EXISTS ( select * from ta_recommend_customer a where a.recommend_person = b.person_id)
             *         </where>
             *         order by b.create_date desc
             */

            // 公客
            IPage<TaPerson>page = new Page<>(pageNumber,pageSize);
            // 公客就不用 buildingId 进行查询
            return taRecommendCustomerMapper.getPublicCustomerList(page,name, tel,  entryType, verifyStatus, sex, orgId,consultTel);
        } else {
            // 私客
            IPage<TaRecommendCustomer>page = new Page<>(pageNumber,pageSize);
            return taRecommendCustomerMapper.getCustomerList(page, building, name, tel, consultName, consultTel, entryType, verifyStatus, sex, orgId,status);
        }

    }

    @Override
    public IPage<TaPerson> getIndependentAgents(int pageNumber, int pageSize,String name,String tel, Integer orgId){
        IPage<TaPerson>page = new Page<>(pageNumber,pageSize);
        return taRecommendCustomerMapper.getIndependentAgents(page,name,tel, orgId);
    }

    @Override
    public IPage<TaRecommendCustomer> getRecCustomerList(int pageNumber, int pageSize,String building,String name,String tel,String consultName,String consultTel,String entryType,String verifyStatus,Integer sex, Integer orgId){
        IPage<TaRecommendCustomer>page = new Page<>(pageNumber,pageSize);
        return taRecommendCustomerMapper.getRecCustomerList(page,building,name,tel,consultName,consultTel,entryType,verifyStatus,sex, orgId);
    }


    @Override
    public IPage<TaRecommendCustomer> getRepCustomerList(int pageNumber, int pageSize,String building,String name,String tel,String consultName,String consultTel,String entryType,String verifyStatus,Integer sex, Integer orgId){
        IPage<TaRecommendCustomer>page = new Page<>(pageNumber,pageSize);
        return taRecommendCustomerMapper.getRepCustomerList(page,building,name,tel,consultName,consultTel,entryType,verifyStatus,sex, orgId);
    }

    @Override
    public TaRecommendCustomer getCustomerById(String customerId){
        return taRecommendCustomerMapper.getCustomerById(customerId);
    }


    @Override
    public TaRecommendCustomer getCustomerDetail(String customerId,int pageNumber, int pageSize){
        String personId = taRecommendCustomerMapper.getCustomerById(customerId).getPersonId();
        TaRecommendCustomer taRecommendCustomer = taRecommendCustomerMapper.getCustomerDetail(customerId,personId);
        TaPerson person = taPersonMapper.selectById(personId);
        if(!StringUtils.isEmpty(personId)){
            IPage<TaPersonVisitRecord>page = new Page<>(pageNumber,pageSize);
            taRecommendCustomer.setVisitRecords(taPersonVisitRecordMapper.visitRecordByPersonId(page,personId));
            taRecommendCustomer.setVisitTimes(taPersonVisitRecordMapper.visitRecordByPersonId(page,personId).getRecords().size());
            taRecommendCustomer.setIntentions(taRecommendCustomerMapper.getCustomerIntentions(personId));
            Map<String,String>geoInfo = new HashMap<>();
            geoInfo.put("provience",person.getProvince());
            geoInfo.put("country",person.getCountry());
            TdCity city = cityMapper.selectById(person.getCity());
            if(null!=city){
                geoInfo.put("city",city.getName());
            }else {
                geoInfo.put("city","未知");
            }
            taRecommendCustomer.setGeoInfo(geoInfo);
        }
        if(!StringUtils.isEmpty(taRecommendCustomer.getRealtyConsultant())){
            TaPerson consultant =  taPersonMapper.getById(taRecommendCustomer.getRealtyConsultant());
            TaUser user = new TaUser();
            if(null == consultant){
                user = userService.getById(taRecommendCustomer.getRealtyConsultant());
            }else {
                user = userService.getById(consultant.getUserId());
            }
            List<String> projects = taPersonMapper.getUserProjects(user.getUserId());
            if (projects.size()>0){
                user.setProjects(projects);
            }
            taRecommendCustomer.setConsultant(user);
        }
        return taRecommendCustomer;
    }

    @Override
    public TaPerson getPublicCustomerDetail(String personId,int pageNumber, int pageSize){
        TaPerson taRecommendCustomer = taPersonMapper.getById(personId);
        if(!StringUtils.isEmpty(personId)){
            IPage<TaPersonVisitRecord>page = new Page<>(pageNumber,pageSize);
            taRecommendCustomer.setVisitRecords(taPersonVisitRecordMapper.visitRecordByPersonId(page,personId));
            taRecommendCustomer.setVisitTimes(taPersonVisitRecordMapper.visitRecordByPersonId(page,personId).getRecords().size());
            taRecommendCustomer.setIntentions(taRecommendCustomerMapper.getCustomerIntentions(personId));
            taRecommendCustomer.setDuration(taPersonVisitRecordMapper.getDurationByPersonId(personId));
            taRecommendCustomer.setVisitTime(taPersonVisitRecordMapper.getFirstVisitTimeByPersonId(personId));
            TdCity city = cityMapper.selectById(taRecommendCustomer.getCity());
            if(null != city){
                taRecommendCustomer.setCity(city.getName());
            }
        }
        return taRecommendCustomer;
    }

    @Override
    public IPage<TaRecommendCustomer>getCustomersIRecommended(int pageNumber, int pageSize,String customerId,Integer orgId){

        QueryWrapper<TaRecommendCustomer>queryWrapper = new QueryWrapper<>();
        // todo
        // 此处可能不是 person_id 而是 recommend_person
        IPage<TaRecommendCustomer>page = new Page<>(pageNumber,pageSize);
//        queryWrapper.eq("recommend_person",customerId);
////        queryWrapper.eq("building_id",building);
//        queryWrapper.eq("status", CommConstant.VERIFY_AGREE);
//        queryWrapper.eq("org_id", orgId);
        return taRecommendCustomerMapper.getCustomersIRecommended(page,customerId,CommConstant.STATUS_NORMAL,orgId);
    }

    @Override
    public TaRecommendCustomer newByPerson(TaPerson person) {
        // 没有手机号, 暂时不生成客户信息
        String phone = StringUtils.ifNull(person.getTel(), person.getPhone());
        if (StringUtils.isEmpty(phone)) {
            return null;
        }

        // 已经存在的, 随机返回一条记录
        List<TaRecommendCustomer> custs = getCustomerByPhone(phone);
        if (null != custs && custs.size() > 0) {
            return custs.get(0);
        }

        TaRecommendCustomer cust = new TaRecommendCustomer();
        cust.setName(StringUtils.ifNull(person.getName(), person.getNickname()));
        cust.setSex(null == person.getSex() ? str2Int(person.getGender()) : person.getSex());
        cust.setPhone(phone);
        cust.setPicture(StringUtils.ifNull(person.getPhoto(), person.getAvatarurl()));
        cust.setCountry(person.getCountry());
        cust.setCity(person.getCity());
        cust.setProvince(person.getProvince());
        // 审核同意
        cust.setVerifyStatus(CommConstant.VERIFY_AGREE);
        // 报备状态
        cust.setStatus(CommConstant.CUSTOMER_REPORT);
        // 当前日期
        cust.setReportDate(LocalDateTime.now());

        if (taRecommendCustomerMapper.insert(cust) > 0) {
            return cust;
        } else {
            log.error("依据人员新增推荐客户失败");
            return null;
        }
    }

    private List<TaRecommendCustomer> getCustomerByPhone(String phone) {
        QueryWrapper<TaRecommendCustomer> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("phone", phone);
        queryWrapper.gt("status", CommConstant.STATUS_DELETE);
        queryWrapper.orderByDesc("report_date");

        List<TaRecommendCustomer> custs = taRecommendCustomerMapper.selectList(queryWrapper);
        return custs;
    }

    private Integer str2Int(String str) {
        if (null == str) {
            return null;
        }

        try {
            return Integer.valueOf(str);
        } catch (Exception e) {
            return 0;
        }
    }


    @Override
    public List<ExcelRecommendCustomer> getRecCustomerExport(Integer orgId, Integer pageCode, Integer pageSize) {
        return taRecommendCustomerMapper.getRecCustomerExport(orgId, pageCode, pageSize);
    }

    @Override
    public Integer getRecCustomerExportCount(Integer orgId) {
        return taRecommendCustomerMapper.getRecCustomerExportCount(orgId);
    }


    @Override
    public Integer getRepCustomerReportCount(String building, String name, String tel, String consultName, String consultTel, String entryType, String verifyStatus, Integer sex,Integer orgId) {
        return taRecommendCustomerMapper.getRepCustomerReportCount(building, name, tel, consultName, consultTel, entryType, verifyStatus, sex, orgId);
    }

    @Override
    public List<ReporRecommendCustomer> getRepCustomerReportExport(String building, String name, String tel, String consultName, String consultTel, String entryType, String verifyStatus, Integer sex, Integer orgId, Integer pageCode, Integer pageSize) {
        return taRecommendCustomerMapper.getRepCustomerReportExport(building, name, tel, consultName, consultTel, entryType,verifyStatus,sex, orgId, pageCode, pageSize);
    }


    @Override
    public Integer getIndependentAgentsCount(Integer orgId) {
        return taRecommendCustomerMapper.getIndependentAgentsCount(orgId);
    }

    @Override
    public List<AgentsRecommendCustomer> getIndependentAgentsExport(Integer orgId, Integer pageCode, Integer pageSize) {
        return taRecommendCustomerMapper.getIndependentAgentsExport(orgId, pageCode, pageSize);
    }
    
    /**
     * 查询我的客户人数集合
     *
     * @param openid
     * @return
     */
    @Override
    public ResponseBean getMyCustStatistics(String openid) {
        List<TaPerson> taPersons = getPersonsByOpenId(openid);
        if (null == taPersons || taPersons.size() != 1) {
            return ResponseBean.error("验证当前人员信息失败", ResponseBean.ERROR_UNAVAILABLE);
        }
    
        TaPerson taPerson = taPersons.get(0);
        Map<String,Object> map = taRecommendCustomerMapper.getMyCustStatistics(taPerson.getPersonId(),taPerson.getUserId(),taPerson.getOrgId());
        return ResponseBean.success(map);
    }
    
    /**
     * 分页查询我的跟进客户集合
     *
     * @param pageNumber
     * @param pageSize
     * @param openid
     * @param name
     * @param phone
     * @param status
     * @param startReportDate
     * @param endReportDate
     * @param startArrivalDate
     * @param endArrivalDate
     * @return
     */
    @Override
    public ResponseBean getMyCustFollowOrClinch(Integer pageNumber, Integer pageSize, String openid, String type, String name, String phone, Integer status, String startReportDate, String endReportDate, String startArrivalDate, String endArrivalDate) {
        List<TaPerson> taPersons = getPersonsByOpenId(openid);
        if (null == taPersons || taPersons.size() != 1) {
            return ResponseBean.error("验证当前人员信息失败", ResponseBean.ERROR_UNAVAILABLE);
        }
    
        TaPerson taPerson = taPersons.get(0);
        
        IPage<TaRecommendCustomer> page = new Page<>();
        page.setCurrent(pageNumber);
        page.setSize(pageSize);
        
        //分页获取跟进客户status != 4的为跟进客户， status = 4的为成交客户 根据type区分  follow 跟进  clinch 成交
        IPage<TaRecommendCustomer> result = taRecommendCustomerMapper.getMyCustFollowOrClinch(page, taPerson.getPersonId(),taPerson.getUserId(),taPerson.getOrgId(), type,name,phone,status,startReportDate,endReportDate,startArrivalDate,endArrivalDate);
        return ResponseBean.success(result);
    }
    
    /**
     * 获取盘客工具下我的客户详情
     *
     * @param customerId
     * @param openid
     * @return
     */
    @Override
    public ResponseBean getMyCustDetail(String customerId, String openid) {
        List<TaPerson> taPersons = getPersonsByOpenId(openid);
        if (null == taPersons || taPersons.size() != 1) {
            return ResponseBean.error("验证当前人员信息失败", ResponseBean.ERROR_UNAVAILABLE);
        }
    
        TaPerson taPerson = taPersons.get(0);
        TaRecommendCustomer taRecommendCustomer = taRecommendCustomerMapper.getMyCustDetailById(customerId);
        //添加跟进行为
        addCustomerFpllowUp(taPerson,taRecommendCustomer,"查看客户详情");
        return ResponseBean.success(taRecommendCustomer);
    }
    
    /**
     * 更新我的客户基本信息
     *
     * @param taRecommendCustomer
     * @param openid
     * @return
     */
    @Override
    public ResponseBean updateMyCust(TaRecommendCustomer taRecommendCustomer, String openid) {
        List<TaPerson> taPersons = getPersonsByOpenId(openid);
        if (null == taPersons || taPersons.size() != 1) {
            return ResponseBean.error("验证当前人员信息失败", ResponseBean.ERROR_UNAVAILABLE);
        }
    
        TaPerson taPerson = taPersons.get(0);
        taRecommendCustomerMapper.updateById(taRecommendCustomer);
        //添加跟进行为
        addCustomerFpllowUp(taPerson,taRecommendCustomer,"编辑客户信息");
        return ResponseBean.success(taRecommendCustomer);
    }
    
    //添加跟进行为
    private void addCustomerFpllowUp(TaPerson taPerson,TaRecommendCustomer taRecommendCustomer,String recordType) {
        TaCustomerFollowUpRecord taCustomerFollowUpRecord = new TaCustomerFollowUpRecord();
        taCustomerFollowUpRecord.setOrgId(taPerson.getOrgId());
        taCustomerFollowUpRecord.setCreateDate(LocalDateTime.now());
        taCustomerFollowUpRecord.setCustomerId(taRecommendCustomer.getCustomerId());
        taCustomerFollowUpRecord.setRecordType(recordType);
        taCustomerFollowUpRecord.setFollowerId(taPerson.getPersonId());
        taCustomerFollowUpRecord.setCustomerSex(taRecommendCustomer.getSex());
        customerFollowUpRecordMapper.insert(taCustomerFollowUpRecord);
    }
}
